@@ -59,12 +59,16 @@
 
 #include "x-c.h"
 #include "x-po.h"
+#include "x-python.h"
 #include "x-lisp.h"
 #include "x-elisp.h"
 #include "x-librep.h"
 #include "x-java.h"
+#include "x-awk.h"
 #include "x-ycp.h"
+#include "x-tcl.h"
 #include "x-rst.h"
+#include "x-glade.h"
 
 
 /* If nonzero add all comments immediately preceding one of the keywords. */
@@ -174,6 +178,7 @@ static void extract_from_file PARAMS ((const char *file_name,
 				       extractor_func extractor,
 				       msgdomain_list_ty *mdlp));
 static message_ty *construct_header PARAMS ((void));
+static void finalize_header PARAMS ((msgdomain_list_ty *mdlp));
 static extractor_func language_to_extractor PARAMS ((const char *name));
 static const char *extension_to_language PARAMS ((const char *extension));
 
@@ -222,10 +227,14 @@ main (argc, argv)
 	break;
       case 'a':
 	x_c_extract_all ();
+	x_python_extract_all ();
 	x_lisp_extract_all ();
 	x_elisp_extract_all ();
 	x_librep_extract_all ();
 	x_java_extract_all ();
+	x_awk_extract_all ();
+	x_tcl_extract_all ();
+	x_glade_extract_all ();
 	break;
       case 'c':
 	if (optarg == NULL)
@@ -276,10 +285,14 @@ main (argc, argv)
 	if (optarg == NULL || *optarg != '\0')
 	  {
 	    x_c_keyword (optarg);
+	    x_python_keyword (optarg);
 	    x_lisp_keyword (optarg);
 	    x_elisp_keyword (optarg);
 	    x_librep_keyword (optarg);
 	    x_java_keyword (optarg);
+	    x_awk_keyword (optarg);
+	    x_tcl_keyword (optarg);
+	    x_glade_keyword (optarg);
 	  }
 	break;
       case 'K':
@@ -486,6 +499,10 @@ warning: file `%s' extension `%s' is unknown; will try C"), fname, extension);
     }
   string_list_free (file_list);
 
+  /* Finalize the constructed header.  */
+  if (!xgettext_omit_header)
+    finalize_header (mdlp);
+
   /* Sorting the list of messages.  */
   if (sort_by_filepos)
     msgdomain_list_sort_by_filepos (mdlp);
@@ -547,8 +564,9 @@ If output file is -, output is written to standard output.\n\
       printf (_("\
 Choice of input file language:\n\
   -L, --language=NAME            recognise the specified language\n\
-                                   (C, C++, ObjectiveC, PO, Lisp, EmacsLisp,\n\
-                                   librep, Java, YCP)\n\
+                                   (C, C++, ObjectiveC, PO, Python, Lisp,\n\
+                                   EmacsLisp, librep, Java, awk, YCP, Tcl,\n\
+                                   RST, Glade)\n\
   -C, --c++                      shorthand for --language=C++\n\
 By default the language is guessed depending on the input file name extension.\n\
 "));
@@ -1163,6 +1181,61 @@ FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.\n");
   return mp;
 }
 
+static void
+finalize_header (mdlp)
+     msgdomain_list_ty *mdlp;
+{
+  /* If the generated PO file has plural forms, add a Plural-Forms template
+     to the constructed header.  */
+  bool has_plural;
+  size_t i, j;
+
+  has_plural = false;
+  for (i = 0; i < mdlp->nitems; i++)
+    {
+      message_list_ty *mlp = mdlp->item[i]->messages;
+
+      for (j = 0; j < mlp->nitems; j++)
+	{
+	  message_ty *mp = mlp->item[j];
+
+	  if (mp->msgid_plural != NULL)
+	    {
+	      has_plural = true;
+	      break;
+	    }
+	}
+      if (has_plural)
+	break;
+    }
+
+  if (has_plural)
+    {
+      message_ty *header = message_list_search (mdlp->item[0]->messages, "");
+      if (header != NULL
+	  && strstr (header->msgstr, "Plural-Forms:") == NULL)
+	{
+	  size_t insertpos = strlen (header->msgstr);
+	  const char *suffix;
+	  size_t suffix_len;
+	  char *new_msgstr;
+
+	  suffix = "\nPlural-Forms: nplurals=INTEGER; plural=EXPRESSION;\n";
+	  if (insertpos == 0 || header->msgstr[insertpos-1] == '\n')
+	    suffix++;
+	  suffix_len = strlen (suffix);
+	  new_msgstr = (char *) xmalloc (header->msgstr_len + suffix_len);
+	  memcpy (new_msgstr, header->msgstr, insertpos);
+	  memcpy (new_msgstr + insertpos, suffix, suffix_len);
+	  memcpy (new_msgstr + insertpos + suffix_len,
+		  header->msgstr + insertpos,
+		  header->msgstr_len - insertpos);
+	  header->msgstr = new_msgstr;
+	  header->msgstr_len = header->msgstr_len + suffix_len;
+	}
+    }
+}
+
 
 #define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
 #define ENDOF(a) ((a) + SIZEOF(a))
@@ -1184,15 +1257,18 @@ language_to_extractor (name)
   {
     SCANNERS_C
     SCANNERS_PO
+    SCANNERS_PYTHON
     SCANNERS_LISP
     SCANNERS_ELISP
     SCANNERS_LIBREP
     SCANNERS_JAVA
+    SCANNERS_AWK
     SCANNERS_YCP
+    SCANNERS_TCL
     SCANNERS_RST
-    { "Python", extract_c, &formatstring_python },
-    /* Here will follow more languages and their scanners: awk, perl,
-       etc...  Make sure new scanners honor the --exclude-file option.  */
+    SCANNERS_GLADE
+    /* Here will follow more languages and their scanners: perl, etc...
+       Make sure new scanners honor the --exclude-file option.  */
   };
 
   table_ty *tp;
@@ -1227,13 +1303,17 @@ extension_to_language (extension)
   {
     EXTENSIONS_C
     EXTENSIONS_PO
+    EXTENSIONS_PYTHON
     EXTENSIONS_LISP
     EXTENSIONS_ELISP
     EXTENSIONS_LIBREP
     EXTENSIONS_JAVA
+    EXTENSIONS_AWK
     EXTENSIONS_YCP
+    EXTENSIONS_TCL
     EXTENSIONS_RST
-    /* Here will follow more file extensions: sh, pl, tcl ... */
+    EXTENSIONS_GLADE
+    /* Here will follow more file extensions: sh, pl ... */
   };
 
   table_ty *tp;
